class ProjectileClientAdjustment expands Info;

var Projectile ControlledProjectile;

// replicated built-in vectors hold inexact values, float values are replicated precisely
struct ReplicatedVector
{
	var float X, Y, Z;
};


var ReplicatedVector ReplicatedLocation;
var ReplicatedVector ReplicatedVelocity;
var ReplicatedVector ReplicatedAcceleration;

replication
{
	reliable if (Role == ROLE_Authority)
		ControlledProjectile,
		ReplicatedLocation,
		ReplicatedVelocity,
		ReplicatedAcceleration;
}

simulated function Tick(float DeltaTime)
{
	if (Level.NetMode != NM_Standalone && ControlledProjectile != none)
	{
		Server_UpdateMovement();
		Client_UpdateMovement();
	}
}

function Server_UpdateMovement()
{
	if (ControlledProjectile == none || ControlledProjectile.bDeleteMe)
	{
		Destroy();
		return;
	}
	ReplicatedLocation = ConvertToReplicatedVector(ControlledProjectile.Location);
	ReplicatedVelocity = ConvertToReplicatedVector(ControlledProjectile.Velocity);
	ReplicatedAcceleration = ConvertToReplicatedVector(ControlledProjectile.Acceleration);
}

simulated function Client_UpdateMovement()
{
	if (Level.NetMode == NM_Client &&
		VSize(ConvertReplicatedVector(ReplicatedLocation)) != 0)
	{
		ControlledProjectile.SetLocation(ConvertReplicatedVector(ReplicatedLocation));
		ControlledProjectile.Velocity = ConvertReplicatedVector(ReplicatedVelocity);
		ControlledProjectile.Acceleration = ConvertReplicatedVector(ReplicatedAcceleration);
		ReplicatedLocation = ConvertToReplicatedVector(vect(0, 0, 0));
	}
}

static function ReplicatedVector ConvertToReplicatedVector(vector pos)
{
	local ReplicatedVector result;
	result.X = pos.X;
	result.Y = pos.Y;
	result.Z = pos.Z;
	return result;
}

static function vector ConvertReplicatedVector(ReplicatedVector pos)
{
	local vector result;
	result.X = pos.X;
	result.Y = pos.Y;
	result.Z = pos.Z;
	return result;
}

defaultproperties
{
				RemoteRole=ROLE_SimulatedProxy
				bAlwaysRelevant=True
}
